@@ -1,6 +1,6 @@
 /* gdaui-entry-string.c
  *
- * Copyright (C) 2003 - 2008 Vivien Malerba
+ * Copyright (C) 2003 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -21,36 +21,35 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "gdaui-entry-string.h"
-#include "gdaui-format-entry.h"
+#include "gdaui-entry.h"
 #include <libgda/gda-data-handler.h>
 #include "gdk/gdkkeysyms.h"
-#include <pango/pango.h>
 
 #define MAX_ACCEPTED_STRING_LENGTH 500
 
 /* 
  * Main static functions 
  */
-static void gdaui_entry_string_class_init (GdauiEntryStringClass * class);
-static void gdaui_entry_string_init (GdauiEntryString * srv);
-static void gdaui_entry_string_dispose (GObject   * object);
-static void gdaui_entry_string_finalize (GObject   * object);
+static void gdaui_entry_string_class_init (GdauiEntryStringClass *klass);
+static void gdaui_entry_string_init (GdauiEntryString *srv);
+static void gdaui_entry_string_dispose (GObject *object);
+static void gdaui_entry_string_finalize (GObject *object);
 
 static void gdaui_entry_string_set_property (GObject *object,
-					  guint param_id,
-					  const GValue *value,
-					  GParamSpec *pspec);
+					     guint param_id,
+					     const GValue *value,
+					     GParamSpec *pspec);
 static void gdaui_entry_string_get_property (GObject *object,
-					  guint param_id,
-					  GValue *value,
-					  GParamSpec *pspec);
+					     guint param_id,
+					     GValue *value,
+					     GParamSpec *pspec);
 
 /* properties */
 enum
 {
 	PROP_0,
 	PROP_MULTILINE,
-	PROP_EDITING_CANCELLED,
+	PROP_EDITING_CANCELED,
 	PROP_OPTIONS
 };
 
@@ -64,7 +63,7 @@ static GtkWidget *create_entry (GdauiEntryWrapper *mgwrap);
 static void       real_set_value (GdauiEntryWrapper *mgwrap, const GValue *value);
 static GValue    *real_get_value (GdauiEntryWrapper *mgwrap);
 static void       connect_signals(GdauiEntryWrapper *mgwrap, GCallback modify_cb, GCallback activate_cb);
-static gboolean   expand_in_layout (GdauiEntryWrapper *mgwrap);
+static gboolean   can_expand (GdauiEntryWrapper *mgwrap, gboolean horiz);
 static void       set_editable (GdauiEntryWrapper *mgwrap, gboolean editable);
 static void       grab_focus (GdauiEntryWrapper *mgwrap);
 
@@ -74,30 +73,21 @@ static void set_entry_options (GdauiEntryString *mgstr, const gchar *options);
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
 
-typedef enum {
-	TYPE_UNKNOWN, /* when value is undefined */
-	TYPE_NUMERIC,
-	TYPE_NOT_NUMERIC
-} InternalType;
-
 /* private structure */
 struct _GdauiEntryStringPrivate
 {
 	gboolean       multiline;
 	gboolean       hidden;
-	InternalType   internal_type;
 	GtkWidget     *vbox;
 
 	GtkWidget     *entry;
+	gboolean       editing_canceled;
 
 	GtkTextBuffer *buffer;
 	GtkWidget     *sw;
 	GtkWidget     *view;
 
 	gint           maxsize;
-	guchar         thousand_sep;
-	gint           nb_decimals;
-	gchar         *currency;
 
 	gulong         entry_change_sig;
 };
@@ -139,22 +129,22 @@ gdaui_entry_string_get_type (void)
 }
 
 static void
-gdaui_entry_string_class_init (GdauiEntryStringClass * class)
+gdaui_entry_string_class_init (GdauiEntryStringClass * klass)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (class);
+	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->dispose = gdaui_entry_string_dispose;
 	object_class->finalize = gdaui_entry_string_finalize;
 
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->create_entry = create_entry;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->real_set_value = real_set_value;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->real_get_value = real_get_value;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->connect_signals = connect_signals;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->expand_in_layout = expand_in_layout;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->set_editable = set_editable;
-	GDAUI_ENTRY_WRAPPER_CLASS (class)->grab_focus = grab_focus;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->create_entry = create_entry;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->real_set_value = real_set_value;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->real_get_value = real_get_value;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->connect_signals = connect_signals;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->can_expand = can_expand;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->set_editable = set_editable;
+	GDAUI_ENTRY_WRAPPER_CLASS (klass)->grab_focus = grab_focus;
 
 	/* Properties */
 	object_class->set_property = gdaui_entry_string_set_property;
@@ -163,53 +153,49 @@ gdaui_entry_string_class_init (GdauiEntryStringClass * class)
 	g_object_class_install_property (object_class, PROP_MULTILINE,
 					 g_param_spec_boolean ("multiline", NULL, NULL, FALSE, 
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (object_class, PROP_EDITING_CANCELLED,
-					 g_param_spec_boolean ("editing_cancelled", NULL, NULL, FALSE, G_PARAM_READABLE));
+	g_object_class_install_property (object_class, PROP_EDITING_CANCELED,
+					 g_param_spec_boolean ("editing-canceled", NULL, NULL, FALSE, G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_OPTIONS,
 					 g_param_spec_string ("options", NULL, NULL, NULL, G_PARAM_WRITABLE));
 }
 
+static gboolean
+key_press_event_cb (GdauiEntryString *mgstr, GdkEventKey *key_event, gpointer data)
+{
+	if (key_event->keyval == GDK_Escape)
+		mgstr->priv->editing_canceled = TRUE;
+	return FALSE;
+}
+
 static void
-gdaui_entry_string_init (GdauiEntryString * mgstr)
+gdaui_entry_string_init (GdauiEntryString *mgstr)
 {
 	mgstr->priv = g_new0 (GdauiEntryStringPrivate, 1);
 	mgstr->priv->multiline = FALSE;
 	mgstr->priv->hidden = FALSE;
 	mgstr->priv->vbox = NULL;
 	mgstr->priv->entry = NULL;
+	mgstr->priv->editing_canceled = FALSE;
 	mgstr->priv->buffer = NULL;
 	mgstr->priv->view = NULL;
 	mgstr->priv->sw = NULL;
 
-	mgstr->priv->internal_type = TYPE_UNKNOWN;
-	mgstr->priv->maxsize = 0; /* unlimited */
-	mgstr->priv->thousand_sep = 0;
-	mgstr->priv->nb_decimals = -1; /* unlimited number of decimals */
-	mgstr->priv->currency = NULL;
+	mgstr->priv->maxsize = 65535; /* eg. unlimited for GtkEntry */
 
 	mgstr->priv->entry_change_sig = 0;
-}
 
-static InternalType
-determine_internal_type (GType type)
-{
-	if ((type == G_TYPE_INT64) || (type == G_TYPE_UINT64) || (type == G_TYPE_DOUBLE) ||
-	    (type == G_TYPE_INT) || (type == GDA_TYPE_NUMERIC) || (type == G_TYPE_FLOAT) ||
-	    (type == GDA_TYPE_SHORT) || (type == GDA_TYPE_USHORT) || (type == G_TYPE_CHAR) ||
-	    (type == G_TYPE_UCHAR) || (type == G_TYPE_LONG) || (type ==  G_TYPE_ULONG) || (type == G_TYPE_UINT))
-		return TYPE_NUMERIC;
-	else
-		return TYPE_NOT_NUMERIC;
+	g_signal_connect (mgstr, "key-press-event",
+			  G_CALLBACK (key_press_event_cb), NULL);
 }
 
 /**
- * gdaui_entry_string_new
+ * gdaui_entry_string_new:
  * @dh: the data handler to be used by the new widget
  * @type: the requested data type (compatible with @dh)
  *
- * Creates a new widget which is mainly a GtkEntry
+ * Creates a new data entry widget
  *
- * Returns: the new widget
+ * Returns: (transfer full): the new widget
  */
 GtkWidget *
 gdaui_entry_string_new (GdaDataHandler *dh, GType type, const gchar *options)
@@ -220,12 +206,12 @@ gdaui_entry_string_new (GdaDataHandler *dh, GType type, const gchar *options)
 	g_return_val_if_fail (GDA_IS_DATA_HANDLER (dh), NULL);
 	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
 	g_return_val_if_fail (gda_data_handler_accepts_g_type (dh, type), NULL);
+	g_return_val_if_fail (type == G_TYPE_STRING, NULL);
 
 	obj = g_object_new (GDAUI_TYPE_ENTRY_STRING, "handler", dh, NULL);
 	mgstr = GDAUI_ENTRY_STRING (obj);
 	gdaui_data_entry_set_value_type (GDAUI_DATA_ENTRY (mgstr), type);
 
-	mgstr->priv->internal_type = determine_internal_type (type);
 	g_object_set (obj, "options", options, NULL);
 
 	return GTK_WIDGET (obj);
@@ -269,7 +255,6 @@ gdaui_entry_string_finalize (GObject   * object)
 
 	mgstr = GDAUI_ENTRY_STRING (object);
 	if (mgstr->priv) {
-		g_free (mgstr->priv->currency);
 		g_free (mgstr->priv);
 		mgstr->priv = NULL;
 	}
@@ -280,9 +265,9 @@ gdaui_entry_string_finalize (GObject   * object)
 
 static void
 gdaui_entry_string_set_property (GObject *object,
-				    guint param_id,
-				    const GValue *value,
-				    GParamSpec *pspec)
+				 guint param_id,
+				 const GValue *value,
+				 GParamSpec *pspec)
 {
 	GdauiEntryString *mgstr;
 
@@ -290,14 +275,6 @@ gdaui_entry_string_set_property (GObject *object,
 	if (mgstr->priv) {
 		switch (param_id) {
 		case PROP_MULTILINE:
-			if (mgstr->priv->internal_type == TYPE_UNKNOWN)
-				mgstr->priv->internal_type = 
-					determine_internal_type (gdaui_data_entry_get_value_type 
-								 (GDAUI_DATA_ENTRY (mgstr)));
-
-			if (mgstr->priv->internal_type == TYPE_NUMERIC)
-				return;
-
 			if (g_value_get_boolean (value) != mgstr->priv->multiline) {
 				mgstr->priv->multiline = g_value_get_boolean (value);
 				if (mgstr->priv->multiline) {
@@ -308,6 +285,7 @@ gdaui_entry_string_set_property (GObject *object,
 					gtk_widget_show (mgstr->priv->entry);
 					gtk_widget_hide (mgstr->priv->sw);
 				}
+				g_signal_emit_by_name (object, "expand-changed");
 			}
 			break;
 		case PROP_OPTIONS:
@@ -322,9 +300,9 @@ gdaui_entry_string_set_property (GObject *object,
 
 static void
 gdaui_entry_string_get_property (GObject *object,
-			     guint param_id,
-			     GValue *value,
-			     GParamSpec *pspec)
+				 guint param_id,
+				 GValue *value,
+				 GParamSpec *pspec)
 {
 	GdauiEntryString *mgstr;
 
@@ -334,8 +312,8 @@ gdaui_entry_string_get_property (GObject *object,
 		case PROP_MULTILINE:
 			g_value_set_boolean (value, mgstr->priv->multiline);
 			break;
-		case PROP_EDITING_CANCELLED:
-			g_value_set_boolean (value, GTK_ENTRY (mgstr->priv->entry)->editing_canceled);
+		case PROP_EDITING_CANCELED:
+			g_value_set_boolean (value, mgstr->priv->editing_canceled);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -360,7 +338,7 @@ create_entry (GdauiEntryWrapper *mgwrap)
 	mgstr->priv->vbox = vbox;
 
 	/* one line entry */
-	mgstr->priv->entry = gdaui_format_entry_new ();
+	mgstr->priv->entry = gdaui_entry_new (NULL, NULL);
 	sync_entry_options (mgstr);
 	gtk_box_pack_start (GTK_BOX (vbox), mgstr->priv->entry, FALSE, TRUE, 0);
 	g_signal_connect_after (G_OBJECT (mgstr->priv->entry), "show", 
@@ -436,12 +414,12 @@ real_set_value (GdauiEntryWrapper *mgwrap, const GValue *value)
 	/* fill the single line widget */
 	if (value) {
 		if (gda_value_is_null ((GValue *) value))
-			gdaui_format_entry_set_text (GDAUI_FORMAT_ENTRY (mgstr->priv->entry), NULL);
+			gdaui_entry_set_text (GDAUI_ENTRY (mgstr->priv->entry), NULL);
 		else 
-			gdaui_format_entry_set_text (GDAUI_FORMAT_ENTRY (mgstr->priv->entry), text);
+			gdaui_entry_set_text (GDAUI_ENTRY (mgstr->priv->entry), text);
 	}
 	else
-		gdaui_format_entry_set_text (GDAUI_FORMAT_ENTRY (mgstr->priv->entry), NULL);
+		gdaui_entry_set_text (GDAUI_ENTRY (mgstr->priv->entry), NULL);
 
 	/* fill the multiline widget */
 	if (value) {
@@ -470,7 +448,7 @@ real_get_value (GdauiEntryWrapper *mgwrap)
 	dh = gdaui_data_entry_get_handler (GDAUI_DATA_ENTRY (mgwrap));
 	if (! mgstr->priv->multiline) {
 		gchar *cstr;
-		cstr = gdaui_format_entry_get_text (GDAUI_FORMAT_ENTRY (mgstr->priv->entry));
+		cstr = gdaui_entry_get_text (GDAUI_ENTRY (mgstr->priv->entry));
 		value = gda_data_handler_get_value_from_str (dh, cstr, gdaui_data_entry_get_value_type (GDAUI_DATA_ENTRY (mgwrap)));
 		g_free (cstr);
 	}
@@ -531,7 +509,7 @@ connect_signals(GdauiEntryWrapper *mgwrap, GCallback modify_cb, GCallback activa
 }
 
 static gboolean
-expand_in_layout (GdauiEntryWrapper *mgwrap)
+can_expand (GdauiEntryWrapper *mgwrap, gboolean horiz)
 {
 	GdauiEntryString *mgstr;
 
@@ -539,7 +517,10 @@ expand_in_layout (GdauiEntryWrapper *mgwrap)
 	mgstr = GDAUI_ENTRY_STRING (mgwrap);
 	g_return_val_if_fail (mgstr->priv, FALSE);
 
-	return mgstr->priv->multiline;
+	if (horiz)
+		return FALSE;
+	else
+		return mgstr->priv->multiline;
 }
 
 static void
@@ -551,7 +532,7 @@ set_editable (GdauiEntryWrapper *mgwrap, gboolean editable)
 	mgstr = GDAUI_ENTRY_STRING (mgwrap);
 	g_return_if_fail (mgstr->priv);
 
-	gtk_entry_set_editable (GTK_ENTRY (mgstr->priv->entry), editable);
+	gtk_editable_set_editable (GTK_EDITABLE (mgstr->priv->entry), editable);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (mgstr->priv->view), editable);
 }
 
@@ -590,11 +571,12 @@ gdaui_entry_string_start_editing (GtkCellEditable *iface, GdkEvent *event)
 {
 	GdauiEntryString *mgstr;
 
-	g_return_if_fail (iface && GDAUI_IS_ENTRY_STRING (iface));
+	g_return_if_fail (GDAUI_IS_ENTRY_STRING (iface));
 	mgstr = GDAUI_ENTRY_STRING (iface);
 	g_return_if_fail (mgstr->priv);
 
-	g_object_set (G_OBJECT (mgstr->priv->entry), "has_frame", FALSE, "xalign", 0., NULL);
+	mgstr->priv->editing_canceled = FALSE;
+	g_object_set (G_OBJECT (mgstr->priv->entry), "has-frame", FALSE, "xalign", 0., NULL);
 	gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (mgstr->priv->view), GTK_TEXT_WINDOW_LEFT, 0);
 	gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (mgstr->priv->view), GTK_TEXT_WINDOW_RIGHT, 0);
 	gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (mgstr->priv->view), GTK_TEXT_WINDOW_TOP, 0);
@@ -603,12 +585,12 @@ gdaui_entry_string_start_editing (GtkCellEditable *iface, GdkEvent *event)
 	gtk_container_set_border_width (GTK_CONTAINER (mgstr->priv->sw), 0);
 
 	gtk_cell_editable_start_editing (GTK_CELL_EDITABLE (mgstr->priv->entry), event);
-	g_signal_connect (G_OBJECT (mgstr->priv->entry), "editing_done",
+	g_signal_connect (mgstr->priv->entry, "editing-done",
 			  G_CALLBACK (gtk_cell_editable_entry_editing_done_cb), mgstr);
-	g_signal_connect (G_OBJECT (mgstr->priv->entry), "remove_widget",
+	g_signal_connect (mgstr->priv->entry, "remove-widget",
 			  G_CALLBACK (gtk_cell_editable_entry_remove_widget_cb), mgstr);
 	gdaui_entry_shell_refresh (GDAUI_ENTRY_SHELL (mgstr));
-
+	
 	gtk_widget_grab_focus (mgstr->priv->entry);
 	gtk_widget_queue_draw (GTK_WIDGET (mgstr));
 }
@@ -616,23 +598,6 @@ gdaui_entry_string_start_editing (GtkCellEditable *iface, GdkEvent *event)
 /*
  * Options handling
  */
-
-static guchar
-get_default_thousands_sep ()
-{
-	static guchar value = 255;
-
-	if (value == 255) {
-		gchar text[20];
-		sprintf (text, "%f", 1234.);
-		if (text[1] == '2')
-			value = 0;
-		else
-			value = text[1];	
-	}
-	return value;
-}
-
 static void
 set_entry_options (GdauiEntryString *mgstr, const gchar *options)
 {
@@ -642,64 +607,39 @@ set_entry_options (GdauiEntryString *mgstr, const gchar *options)
                 GdaQuarkList *params;
                 const gchar *str;
 		
-		if (mgstr->priv->internal_type == TYPE_UNKNOWN)
-			mgstr->priv->internal_type = 
-				determine_internal_type (gdaui_data_entry_get_value_type 
-							 (GDAUI_DATA_ENTRY (mgstr)));
-
                 params = gda_quark_list_new_from_string (options);
-		if (mgstr->priv->internal_type == TYPE_NOT_NUMERIC) {
-			/* STRING specific options */
-			str = gda_quark_list_find (params, "MAX_SIZE");
-			if (str) 
-				mgstr->priv->maxsize = atoi (str);
 
-			str = gda_quark_list_find (params, "MULTILINE");
-			if (str) {
-				if ((*str == 't') || (*str == 'T'))
-					mgstr->priv->multiline = TRUE;
-				else
-					mgstr->priv->multiline = FALSE;
-			       
+		str = gda_quark_list_find (params, "MAX_SIZE");
+		if (str) 
+			mgstr->priv->maxsize = atoi (str);
+		
+		str = gda_quark_list_find (params, "MULTILINE");
+		if (str) {
+			if ((*str == 't') || (*str == 'T'))
+				mgstr->priv->multiline = TRUE;
+			else
+				mgstr->priv->multiline = FALSE;
+			
+		}
+		
+		str = gda_quark_list_find (params, "HIDDEN");
+		if (str) {
+			if ((*str == 't') || (*str == 'T'))
+				mgstr->priv->hidden = TRUE;
+			else
+				mgstr->priv->hidden = FALSE;
+		}
+		
+		if (mgstr->priv->entry) {
+			if (mgstr->priv->multiline) {
+				gtk_widget_hide (mgstr->priv->entry);
+				gtk_widget_show (mgstr->priv->sw);
 			}
-
-			str = gda_quark_list_find (params, "HIDDEN");
-			if (str) {
-				if ((*str == 't') || (*str == 'T'))
-					mgstr->priv->hidden = TRUE;
-				else
-					mgstr->priv->hidden = FALSE;
-			}
-
-			if (mgstr->priv->entry) {
-				if (mgstr->priv->multiline) {
-					gtk_widget_hide (mgstr->priv->entry);
-					gtk_widget_show (mgstr->priv->sw);
-				}
-				else {
-					gtk_widget_show (mgstr->priv->entry);
-					gtk_widget_hide (mgstr->priv->sw);
-					gtk_entry_set_visibility (GTK_ENTRY (mgstr->priv->entry), 
-								  !mgstr->priv->hidden);
-				}
-			}
-                }
-		else {
-			/* NUMERIC specific options */
-			str = gda_quark_list_find (params, "THOUSAND_SEP");
-			if (str) {
-				if ((*str == 't') || (*str == 'T'))
-					mgstr->priv->thousand_sep = get_default_thousands_sep ();
-				else
-					mgstr->priv->thousand_sep = 0;
-			}
-			str = gda_quark_list_find (params, "NB_DECIMALS");
-			if (str) 
-				mgstr->priv->nb_decimals = atoi (str);
-			str = gda_quark_list_find (params, "CURRENCY");
-			if (str) {
-				g_free (mgstr->priv->currency);
-				mgstr->priv->currency = g_strdup_printf ("%s ", str);
+			else {
+				gtk_widget_show (mgstr->priv->entry);
+				gtk_widget_hide (mgstr->priv->sw);
+				gtk_entry_set_visibility (GTK_ENTRY (mgstr->priv->entry), 
+							  !mgstr->priv->hidden);
 			}
 		}
                 gda_quark_list_free (params);
@@ -715,11 +655,7 @@ sync_entry_options (GdauiEntryString *mgstr)
 		return;
 
 	g_object_set (G_OBJECT (mgstr->priv->entry), 
-		      "edited_type", gdaui_data_entry_get_value_type (GDAUI_DATA_ENTRY (mgstr)),
-		      "n_decimals", mgstr->priv->nb_decimals,
-		      "thousands_sep", mgstr->priv->thousand_sep,
-		      "prefix", mgstr->priv->currency,
-		      "max_length", mgstr->priv->maxsize,
+		      "max-length", mgstr->priv->maxsize,
 		      NULL);
 	g_signal_emit_by_name (mgstr->priv->entry, "changed");
 }
