@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2009 The GNOME Foundation.
+ * Copyright (C) 1999 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -29,31 +29,42 @@
 #include <libgda-ui/gdaui-plugin.h>
 #include <libgda/binreloc/gda-binreloc.h>
 #include "data-entries/gdaui-entry-boolean.h"
+#include "data-entries/gdaui-entry-bin.h"
 #include "data-entries/gdaui-entry-string.h"
+#include "data-entries/gdaui-entry-number.h"
 #include "data-entries/gdaui-entry-time.h"
 #include "data-entries/gdaui-entry-date.h"
 #include "data-entries/gdaui-entry-timestamp.h"
 #include "data-entries/gdaui-entry-none.h"
 #include "data-entries/gdaui-data-cell-renderer-textual.h"
 #include "data-entries/gdaui-data-cell-renderer-boolean.h"
+#include "data-entries/gdaui-data-cell-renderer-bin.h"
 
 /* plugins list */
 
+typedef GSList           *(*GdauiPluginInit)     (GError **);
 static GHashTable *init_plugins_hash (void);
 GHashTable *gdaui_plugins_hash = NULL; /* key = plugin name, value = GdauiPlugin structure pointer */
 
 
 /**
- * gdaui_init
+ * gdaui_init:
  *
  * Initialization of the libgda-ui library, must be called before any usage of the library.
- * Note: gtk_init() is not called by this function and should also be called.
+ *
+ * <itemizedlist>
+ * <listitem><para>Note1: gtk_init() is not called by this function and should also
+ *   be called (the calling order has no importance)</para></listitem>
+ * <listitem><para>Note2: this funtion also calls gda_init() so it should not be called
+ *    again</para></listitem>
+ * </itemizedlist>
+ *
+ * Since: 4.2
  */
 void
 gdaui_init (void)
 {
 	static gboolean initialized = FALSE;
-	gchar *str;
 
 	if (initialized) {
 		gda_log_error (_("Attempt to initialize an already initialized library"));
@@ -61,6 +72,7 @@ gdaui_init (void)
 	}
 
 	/*
+	gchar *str;
 	gdaui_gbr_init ();
 	str = gdaui_gbr_get_locale_dir_path ();
 	bindtextdomain (GETTEXT_PACKAGE, str);
@@ -70,13 +82,14 @@ gdaui_init (void)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	gda_init ();
-	gdaui_plugins_hash = init_plugins_hash ();
+	if (! gdaui_plugins_hash)
+		gdaui_plugins_hash = init_plugins_hash ();
 
 	initialized = TRUE;
 }
 
 /**
- * gdaui_new_data_entry
+ * gdaui_new_data_entry:
  * @type: a #GType
  * @plugin_name: the name of an entry plugin, or %NULL
  *
@@ -87,7 +100,7 @@ gdaui_init (void)
  * @plugin_name format is interpreted as two parts: &lt;plugin name&gt;:&lt;plugin options&gt;, and
  * if the plugins has no option, then the ":&lt;plugin options&gt;" part may be omitted.
  * 
- * Returns: a new #GdauiDataEntry widget, _NEVER_ %NULL
+ * Returns: (transfer full): a new #GdauiDataEntry widget, _NEVER_ %NULL
  */
 GdauiDataEntry *
 gdaui_new_data_entry (GType type, const gchar *plugin_name)
@@ -99,6 +112,9 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 	if (!gdaui_plugins_hash)
 		gdaui_plugins_hash = init_plugins_hash ();
 
+	if (type == G_TYPE_INVALID)
+		return (GdauiDataEntry *) gdaui_entry_none_new (GDA_TYPE_NULL);
+
 	dh = gda_get_default_handler (type);
 
 	if (plugin_name && *plugin_name) {
@@ -121,28 +137,29 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 	}
 
 	if (!entry) {
-		if (type == GDA_TYPE_NULL)
-			entry = (GdauiDataEntry *) gdaui_entry_none_new (GDA_TYPE_NULL);
+		if (type == G_TYPE_STRING)
+			entry = (GdauiDataEntry *) gdaui_entry_string_new (dh, type, spec_options);
 		else if ((type == G_TYPE_INT64) ||
 			 (type == G_TYPE_UINT64) ||
-			 (type == GDA_TYPE_BINARY) ||
 			 (type == G_TYPE_DOUBLE) ||
 			 (type == G_TYPE_INT) ||
+			 (type == G_TYPE_LONG) ||
 			 (type == GDA_TYPE_NUMERIC) ||
 			 (type == G_TYPE_FLOAT) ||
 			 (type == GDA_TYPE_SHORT) ||
 			 (type == GDA_TYPE_USHORT) ||
-			 (type == G_TYPE_STRING) ||
 			 (type == G_TYPE_CHAR) ||
 			 (type == G_TYPE_UCHAR) ||
 			 (type == G_TYPE_ULONG) ||
 			 (type == G_TYPE_UINT))
-			entry = (GdauiDataEntry *) gdaui_entry_string_new (dh, type, spec_options);
+			entry = (GdauiDataEntry *) gdaui_entry_number_new (dh, type, spec_options);
 		else if (type == G_TYPE_BOOLEAN)
 			entry = (GdauiDataEntry *) gdaui_entry_boolean_new (dh, G_TYPE_BOOLEAN);
+		else if ((type == GDA_TYPE_BLOB) ||
+			 (type == GDA_TYPE_BINARY))
+			entry = (GdauiDataEntry *) gdaui_entry_bin_new (dh, type);
 		else if	((type == GDA_TYPE_GEOMETRIC_POINT) ||
 			 (type == G_TYPE_OBJECT) ||
-			 (type == GDA_TYPE_BLOB) ||
 			 (type == GDA_TYPE_LIST))
 			entry = (GdauiDataEntry *) gdaui_entry_none_new (type);
 		else if	(type == GDA_TYPE_TIME)
@@ -152,15 +169,15 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 		else if (type == G_TYPE_DATE)
 			entry = (GdauiDataEntry *) gdaui_entry_date_new (dh);
 		else
-			entry = (GdauiDataEntry *) gdaui_entry_string_new (dh, type, spec_options);
+			entry = (GdauiDataEntry *) gdaui_entry_none_new (type);
 	}
 
 	g_free (spec_options);
 	return entry;
 }
 
-/**
- * gdaui_new_cell_renderer
+/*
+ * _gdaui_new_cell_renderer
  * @type: a #GType
  * @plugin_name: the name of an entry plugin, or %NULL
  *
@@ -176,7 +193,7 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
  * Returns: a new #GtkCellRenderer object, _NEVER_ %NULL
  */
 GtkCellRenderer *
-gdaui_new_cell_renderer (GType type, const gchar *plugin_name)
+_gdaui_new_cell_renderer (GType type, const gchar *plugin_name)
 {
 	GdaDataHandler *dh;
 	GtkCellRenderer *cell = NULL;
@@ -208,6 +225,9 @@ gdaui_new_cell_renderer (GType type, const gchar *plugin_name)
 			cell = gdaui_data_cell_renderer_textual_new (NULL, GDA_TYPE_NULL, NULL);
 		else if (type == G_TYPE_BOOLEAN)
 			cell = gdaui_data_cell_renderer_boolean_new (dh, G_TYPE_BOOLEAN);
+		else if ((type == GDA_TYPE_BLOB) ||
+			 (type == GDA_TYPE_BINARY))
+			cell = gdaui_data_cell_renderer_bin_new (dh, type);
 		else
 			cell = gdaui_data_cell_renderer_textual_new (dh, type, NULL);
 	}
@@ -215,15 +235,68 @@ gdaui_new_cell_renderer (GType type, const gchar *plugin_name)
 	return cell;
 }
 
+/**
+ * gdaui_plugin_declare:
+ * @plugin: a pointer to a structure filled to describe the new plugin
+ *
+ * Adds a new plugin which will be used by the forms and grids. The new plugin, as
+ * described by @plugin can declare a custom widget to be used for forms, grids, or both.
+ *
+ * If a plugin is already declared with the same name as the requested name, then
+ * a warning is issued and the operation fails.
+ */
+void
+gdaui_plugin_declare (const GdauiPlugin *plugin)
+{
+	GdauiPlugin *np;
+
+	g_return_if_fail (plugin);
+	g_return_if_fail (plugin->plugin_name);
+	if (!gdaui_plugins_hash)
+		gdaui_plugins_hash = init_plugins_hash ();
+	if (g_hash_table_lookup (gdaui_plugins_hash, plugin->plugin_name)) {
+		g_warning ("Plugin '%s' already declared", plugin->plugin_name);
+		return;
+	}
+	if (((plugin->nb_g_types < 1) && plugin->valid_g_types) ||
+	    ((plugin->nb_g_types > 0) && !plugin->valid_g_types)) {
+		g_warning ("Invalid description of plugin accepted types");
+		return;
+	}
+	g_return_if_fail (plugin->entry_create_func || plugin->cell_create_func);
+	
+	np = g_new0 (GdauiPlugin, 1);
+	np->plugin_name = g_strdup (plugin->plugin_name);
+	if (plugin->plugin_descr)
+		np->plugin_descr = g_strdup (plugin->plugin_descr);
+	np->plugin_file = g_strdup (plugin->plugin_file);
+
+	np->nb_g_types = plugin->nb_g_types;
+	if (plugin->valid_g_types) {
+		np->valid_g_types = g_new0 (GType, np->nb_g_types);
+		memcpy (np->valid_g_types, plugin->valid_g_types, np->nb_g_types);
+	}
+
+	if (plugin->options_xml_spec)
+		np->options_xml_spec = g_strdup (plugin->options_xml_spec);
+	np->entry_create_func = plugin->entry_create_func;
+	np->cell_create_func = plugin->cell_create_func;
+
+	g_hash_table_insert (gdaui_plugins_hash, np->plugin_name, np);
+}
+
 static GdauiDataEntry *entry_none_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GdauiDataEntry *entry_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *options);
+static GdauiDataEntry *entry_bin_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GdauiDataEntry *entry_string_create_func (GdaDataHandler *handler, GType type, const gchar *options);
+static GdauiDataEntry *entry_number_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GdauiDataEntry *entry_time_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GdauiDataEntry *entry_timestamp_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GdauiDataEntry *entry_date_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 
 static GtkCellRenderer *cell_textual_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 static GtkCellRenderer *cell_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *options);
+static GtkCellRenderer *cell_bin_create_func (GdaDataHandler *handler, GType type, const gchar *options);
 
 static xmlChar *get_spec_with_isocodes (const gchar *file);
 
@@ -257,7 +330,20 @@ init_plugins_hash (void)
 	plugin->valid_g_types [0] = G_TYPE_BOOLEAN;
 	plugin->options_xml_spec = NULL;
 	plugin->entry_create_func = entry_boolean_create_func;
-	plugin->cell_create_func =cell_boolean_create_func;
+	plugin->cell_create_func = cell_boolean_create_func;
+	g_hash_table_insert (hash, plugin->plugin_name, plugin);
+
+	plugin = g_new0 (GdauiPlugin, 1);
+	plugin->plugin_name = "binary";
+	plugin->plugin_descr = "Binary data entry";
+	plugin->plugin_file = NULL;
+	plugin->nb_g_types = 2;
+	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
+	plugin->valid_g_types [0] = GDA_TYPE_BLOB;
+	plugin->valid_g_types [1] = GDA_TYPE_BINARY;
+	plugin->options_xml_spec = NULL;
+	plugin->entry_create_func = entry_bin_create_func;
+	plugin->cell_create_func = cell_bin_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
        
 	plugin = g_new0 (GdauiPlugin, 1);
@@ -271,7 +357,7 @@ init_plugins_hash (void)
 	plugin->entry_create_func = entry_string_create_func;
 	plugin->cell_create_func = cell_textual_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
-	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-string-string.xml", NULL);
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-string.xml", NULL);
 	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
 		g_message ("Could not find file '%s': '%s' data entry will not report any possible option",
 			   file, plugin->plugin_name);
@@ -301,13 +387,13 @@ init_plugins_hash (void)
 	plugin->valid_g_types [10] = G_TYPE_ULONG;
 	plugin->valid_g_types [11] = G_TYPE_UINT;
 	plugin->options_xml_spec = NULL;
-	plugin->entry_create_func = entry_string_create_func;
+	plugin->entry_create_func = entry_number_create_func;
 	plugin->cell_create_func = cell_textual_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
-	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-string-number.xml", NULL);
+	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-number.xml", NULL);
 	xmlChar *xml_spec = get_spec_with_isocodes (file);
 	if (xml_spec) {
-		plugin->options_xml_spec = g_strdup (xml_spec);
+		plugin->options_xml_spec = g_strdup ((gchar*) xml_spec);
 		xmlFree (xml_spec);
 	}
 	g_free (file);
@@ -316,8 +402,9 @@ init_plugins_hash (void)
 	plugin->plugin_name = "textual";
 	plugin->plugin_descr = "Textual entry";
 	plugin->plugin_file = NULL;
-	plugin->nb_g_types = 0;
-	plugin->valid_g_types = NULL;
+	plugin->nb_g_types = 1;
+	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
+	plugin->valid_g_types [0] = G_TYPE_STRING;
 	plugin->options_xml_spec = NULL;
 	plugin->entry_create_func = entry_string_create_func;
 	plugin->cell_create_func = cell_textual_create_func;
@@ -450,12 +537,24 @@ entry_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *opt
 	return (GdauiDataEntry *) gdaui_entry_boolean_new (handler, G_TYPE_BOOLEAN);
 }
 
+static GdauiDataEntry *
+entry_bin_create_func (GdaDataHandler *handler, GType type, const gchar *options)
+{
+	return (GdauiDataEntry *) gdaui_entry_bin_new (handler, type);
+}
+
 static GdauiDataEntry *
 entry_string_create_func (GdaDataHandler *handler, GType type, const gchar *options)
 {
 	return (GdauiDataEntry *) gdaui_entry_string_new (handler, type, options);
 }
 
+static GdauiDataEntry *
+entry_number_create_func (GdaDataHandler *handler, GType type, const gchar *options)
+{
+	return (GdauiDataEntry *) gdaui_entry_number_new (handler, type, options);
+}
+
 static GdauiDataEntry *
 entry_time_create_func (GdaDataHandler *handler, GType type, const gchar *options)
 {
@@ -486,6 +585,12 @@ cell_boolean_create_func (GdaDataHandler *handler, GType type, const gchar *opti
 	return gdaui_data_cell_renderer_boolean_new (handler, G_TYPE_BOOLEAN);
 }
 
+static GtkCellRenderer *
+cell_bin_create_func (GdaDataHandler *handler, GType type, const gchar *options)
+{
+	return gdaui_data_cell_renderer_bin_new (handler, type);
+}
+
 static xmlNodePtr
 find_child_node_from_name (xmlNodePtr parent, const gchar *name, const gchar *attr_name, const gchar *attr_value)
 {
@@ -495,12 +600,12 @@ find_child_node_from_name (xmlNodePtr parent, const gchar *name, const gchar *at
 		return NULL;
 
 	for (node = parent->children; node; node = node->next) {
-		if (!strcmp (node->name, name)) {
+		if (!strcmp ((gchar*) node->name, name)) {
 			if (attr_name) {
 				xmlChar *prop;
-				prop = xmlGetProp (node, attr_name);
+				prop = xmlGetProp (node, BAD_CAST attr_name);
 				if (prop) {
-					if (attr_value && !strcmp (prop, attr_value)) {
+					if (attr_value && !strcmp ((gchar*) prop, attr_value)) {
 						xmlFree (prop);
 						break;
 					}
@@ -521,7 +626,7 @@ static xmlChar *
 get_spec_with_isocodes (const gchar *file)
 {
 	xmlDocPtr spec, isocodes = NULL;
-	gchar *retval = NULL;
+	xmlChar *retval = NULL;
 	gchar *isofile = NULL;
 	GError *err = NULL;
 	gchar  *buf = NULL;
@@ -562,20 +667,21 @@ get_spec_with_isocodes (const gchar *file)
 		spec_node = find_child_node_from_name (node, "gda_array_data", NULL, NULL);
 		xmlUnlinkNode (spec_node);
 		xmlFreeNode (spec_node);
-		spec_node = xmlNewChild (node, NULL, "gda_array_data", NULL);
+		spec_node = xmlNewChild (node, NULL, BAD_CAST "gda_array_data", NULL);
 		
 		node = xmlDocGetRootElement (isocodes);
 		for (node = node->children; node; node = node->next) {
-			if (!strcmp (node->name, "iso_4217_entry")) {
+			if (!strcmp ((gchar*) node->name, "iso_4217_entry")) {
 				xmlChar *code, *name;
-				code = xmlGetProp (node, "letter_code");
-				name = xmlGetProp (node, "currency_name");
+				code = xmlGetProp (node, BAD_CAST "letter_code");
+				name = xmlGetProp (node, BAD_CAST "currency_name");
 				if (code && name) {
 					xmlNodePtr row;
-					row = xmlNewChild (spec_node, NULL, "gda_array_row", NULL);
-					xmlNewChild (row, NULL, "gda_value", code);
-					xmlNewChild (row, NULL, "gda_value", code);
-					xmlNewChild (row, NULL, "gda_value", dgettext ("iso_4217", name));
+					row = xmlNewChild (spec_node, NULL, BAD_CAST "gda_array_row", NULL);
+					xmlNewChild (row, NULL, BAD_CAST "gda_value", code);
+					xmlNewChild (row, NULL, BAD_CAST "gda_value", code);
+					xmlNewChild (row, NULL, BAD_CAST "gda_value",
+						     BAD_CAST dgettext ("iso_4217", (gchar*) name));
 				}
 				if (code)
 					xmlFree (code);
@@ -596,7 +702,7 @@ get_spec_with_isocodes (const gchar *file)
 
 		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameters", NULL, NULL);
 		node = find_child_node_from_name (xmlDocGetRootElement (spec), "parameter", "id", "CURRENCY");
-		xmlSetProp (node, "source", NULL);
+		xmlSetProp (node, BAD_CAST "source", NULL);
 	}
 
 	xmlDocDumpMemory (spec, (xmlChar **) &retval, &buf_len);
