@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2008 The GNOME Foundation.
+ * Copyright (C) 1999 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -30,9 +30,7 @@
 #include <libgda-ui/internal/gdaui-provider-spec-editor.h>
 #include <libgda-ui/internal/gdaui-provider-auth-editor.h>
 #include "gdaui-enum-types.h"
-#include <gtk/gtklabel.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 #include <string.h>
 #include <libgda/binreloc/gda-binreloc.h>
 
@@ -130,6 +128,19 @@ gdaui_login_class_init (GdauiLoginClass *klass)
 							       FALSE, G_PARAM_READABLE));
 }
 
+static void
+config_dsn_changed_cb (GdaConfig *config, GdaDsnInfo *dsn, GdauiLogin *login)
+{
+	if (!login->priv->prov_selector)
+		return;
+	gchar *sdsn;
+	sdsn = _gdaui_dsn_selector_get_dsn (GDAUI_DSN_SELECTOR (login->priv->dsn_selector));
+	if (dsn && dsn->name && sdsn && !strcmp (dsn->name, sdsn)) {
+		dsn_entry_changed_cb (GDAUI_DSN_SELECTOR (login->priv->dsn_selector), login);
+		g_print ("Update...\n");
+	}
+}
+
 static void
 gdaui_login_init (GdauiLogin *login, GdauiLoginClass *klass)
 {
@@ -143,6 +154,10 @@ gdaui_login_init (GdauiLogin *login, GdauiLoginClass *klass)
 	login->priv->mode = GDA_UI_LOGIN_ENABLE_CONTROL_CENTRE_MODE;
 	memset (&(login->priv->dsn_info), 0, sizeof (GdaDsnInfo));
 	
+	/* catch DSN definition changes */
+	g_signal_connect (gda_config_get (), "dsn-changed",
+			  G_CALLBACK (config_dsn_changed_cb), login);
+
 	/* table layout */
 	table = gtk_table_new (3, 3, FALSE);
 	gtk_widget_show (table);
@@ -283,6 +298,9 @@ gdaui_login_finalize (GObject *object)
 
 	g_return_if_fail (GDAUI_IS_LOGIN (login));
 
+	g_signal_handlers_disconnect_by_func (gda_config_get (),
+					      G_CALLBACK (config_dsn_changed_cb), login);
+
 	/* free memory */
 	clear_dsn_info (login);
 	g_free (login->priv);
@@ -390,7 +408,11 @@ run_cc_cb (GtkButton *button, GdauiLogin *login)
 	if (!sresult) {
 		GtkWidget *msgdialog;
 		GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (login));
+#if GTK_CHECK_VERSION(2,18,0)
+		if (!gtk_widget_is_toplevel (toplevel))
+#else
 		if (!GTK_WIDGET_TOPLEVEL (toplevel))
+#endif
 			toplevel = NULL;
 		msgdialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (toplevel), GTK_DIALOG_MODAL,
 								GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -464,12 +486,12 @@ cnc_params_editor_changed_cb (GdauiProviderSpecEditor *editor, GdauiLogin *login
 }
 
 /**
- * gdaui_login_new
+ * gdaui_login_new:
  * @dsn: a data source name, or %NULL
  *
  * Creates a new login widget which enables the user to specify connection parameters.
  *
- * Returns: a new widget
+ * Returns: (transfer full): a new widget
   *
  * Since: 4.2
 */
@@ -484,7 +506,7 @@ gdaui_login_new (const gchar *dsn)
 
 
 /**
- * gdaui_login_set_mode
+ * gdaui_login_set_mode:
  * @login: a #GdauiLogin object
  * @mode: a flag
  *
@@ -537,14 +559,14 @@ gdaui_login_set_mode (GdauiLogin *login, GdauiLoginMode mode)
 }
 
 /**
- * gdaui_login_get_connection_information
+ * gdaui_login_get_connection_information:
  * @login: a #GdauiLogin object
  *
  * Get the information specified in @login as a pointer to a (read-only) #GdaDsnInfo.
  * If the connection is not specified by a DSN, then the 'name' attribute of the returned
  * #GdaDsnInfo will be %NULL, and otherwise it will contain the name of the selected DSN.
  *
- * Retuns: a pointer to a (read-only) #GdaDsnInfo.
+ * Retuns: (transfer none): a pointer to a (read-only) #GdaDsnInfo.
  *
  * Since: 4.2
  */
@@ -555,24 +577,23 @@ gdaui_login_get_connection_information (GdauiLogin *login)
 
 	clear_dsn_info (login);
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (login->priv->rb_dsn))) {
-		GdaDsnInfo *info;
+		GdaDsnInfo *info = NULL;
 		gchar *dsn;
 		dsn = _gdaui_dsn_selector_get_dsn (GDAUI_DSN_SELECTOR (login->priv->dsn_selector));
-		info = gda_config_get_dsn_info (dsn);
 		if (dsn && *dsn)
-			login->priv->dsn_info.name = dsn;
-		else
-			g_free (dsn);
+			info = gda_config_get_dsn_info (dsn);
+		g_free (dsn);
 		if (info) {
+			login->priv->dsn_info.name = g_strdup (info->name);
 			if (info->provider)
 				login->priv->dsn_info.provider = g_strdup (info->provider);
 			if (info->description)
 				login->priv->dsn_info.description = g_strdup (info->description);
 			if (info->cnc_string)
 				login->priv->dsn_info.cnc_string = g_strdup (info->cnc_string);
-			login->priv->dsn_info.auth_string = _gdaui_provider_auth_editor_get_auth (GDAUI_PROVIDER_AUTH_EDITOR (login->priv->auth_widget));
 			login->priv->dsn_info.is_system = info->is_system;
 		}
+		login->priv->dsn_info.auth_string = _gdaui_provider_auth_editor_get_auth (GDAUI_PROVIDER_AUTH_EDITOR (login->priv->auth_widget));
 	}
 	else {
 		const gchar *str;
@@ -587,7 +608,7 @@ gdaui_login_get_connection_information (GdauiLogin *login)
 }
 
 /**
- * gdaui_login_set_dsn
+ * gdaui_login_set_dsn:
  * @login: a #GdauiLogin object
  * @dsn: a data source name, or %NULL
  *
@@ -613,7 +634,7 @@ gdaui_login_set_dsn (GdauiLogin *login, const gchar *dsn)
 }
 
 /**
- * gdaui_login_set_connection_information
+ * gdaui_login_set_connection_information:
  * @login: a #GdauiLogin object
  * @cinfo: a pointer to a structure representing the information to display.
  *
@@ -633,7 +654,22 @@ gdaui_login_set_connection_information (GdauiLogin *login, const GdaDsnInfo *cin
 		_gdaui_dsn_selector_set_dsn (GDAUI_DSN_SELECTOR (login->priv->dsn_selector), NULL);
 	}
 	else {
-		_gdaui_dsn_selector_set_dsn (GDAUI_DSN_SELECTOR (login->priv->dsn_selector), cinfo->name);
+		GdaDsnInfo *info = NULL;
+		if (cinfo->name)
+			info = gda_config_get_dsn_info (cinfo->name);
+		if (info)
+			_gdaui_dsn_selector_set_dsn (GDAUI_DSN_SELECTOR (login->priv->dsn_selector), cinfo->name);
+		else {
+			/* force switch to */
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (login->priv->rb_dsn), FALSE);
+		}
+		g_signal_handlers_block_by_func (login->priv->prov_selector,
+						 G_CALLBACK (prov_entry_changed_cb), login);
+		gdaui_provider_selector_set_provider (GDAUI_PROVIDER_SELECTOR (login->priv->prov_selector),
+		                                      cinfo->provider);
+		g_signal_handlers_unblock_by_func (login->priv->prov_selector,
+						   G_CALLBACK (prov_entry_changed_cb), login);
+
 		_gdaui_provider_spec_editor_set_provider (GDAUI_PROVIDER_SPEC_EDITOR (login->priv->cnc_params_editor),
 							  cinfo->provider);
 		_gdaui_provider_spec_editor_set_specs (GDAUI_PROVIDER_SPEC_EDITOR (login->priv->cnc_params_editor),
