@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 2007 - 2009 The GNOME Foundation
+ * Copyright (C) 2007 - 2010 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -29,10 +29,6 @@
 
 #ifdef HAVE_GIO
 #include <gio/gio.h>
-#else
-  #ifdef HAVE_GNOMEVFS
-  #include <libgnomevfs/gnome-vfs-mime.h>
-  #endif
 #endif
 
 /* Use the RSA reference implementation included in the RFC-1321, http://www.freesoft.org/CIE/RFC/1321/ */
@@ -597,19 +593,7 @@ update_file_mime (FileRow *row, const gchar *complete_filename)
 		value = gda_value_new_null ();
 	g_object_unref (file);
 #else
-  #ifdef HAVE_GNOMEVFS
-	const gchar *mime;
-	
-	mime = gnome_vfs_get_file_mime_type (complete_filename, NULL, FALSE);
-	if (mime) {
-		value = gda_value_new (G_TYPE_STRING);
-		g_value_set_string (value, mime);
-	}
-	else 
-		value = gda_value_new_null ();
-  #else
 	value = gda_value_new_null ();
-  #endif
 #endif
 
 	if (value) {
@@ -670,7 +654,7 @@ gda_data_model_dir_get_property (GObject *object,
 }
 
 /**
- * gda_data_model_dir_new
+ * gda_data_model_dir_new:
  * @basedir: a directory
  *
  * Creates a new #GdaDataModel object to list all the files starting from @basedir
@@ -690,12 +674,12 @@ gda_data_model_dir_new (const gchar *basedir)
 }
 
 /**
- * gda_data_model_dir_get_errors
+ * gda_data_model_dir_get_errors:
  * @model: a #GdaDataModelDir object
  *
  * Get the list of errors which have occurred while using @model
  *
- * Returns: a read-only list of #GError pointers, or %NULL if no error has occurred
+ * Returns: (transfer none) (element-type GObject.Error) : a read-only list of #GError pointers, or %NULL if no error has occurred
  */
 const GSList *
 gda_data_model_dir_get_errors (GdaDataModelDir *model)
@@ -707,7 +691,7 @@ gda_data_model_dir_get_errors (GdaDataModelDir *model)
 }
 
 /**
- * gda_data_model_dir_clean_errors
+ * gda_data_model_dir_clean_errors:
  * @model: a #GdaDataModelDir object
  *
  * Reset the list of errors which have occurred while using @model
