@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+ * Copyright (C) 1998 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -73,7 +73,7 @@ gda_row_class_init (GdaRowClass *klass)
         object_class->get_property = gda_row_get_property;
 
 	g_object_class_install_property (object_class, PROP_NB_VALUES,
-                                         g_param_spec_int ("nb-values", NULL, NULL,
+                                         g_param_spec_int ("nb-values", NULL, "Number of values in the row",
 							   1, G_MAXINT, 1, 
 							   G_PARAM_WRITABLE));
 }
@@ -193,16 +193,11 @@ gda_row_get_type (void)
 }
 
 /**
- * gda_row_new
- * @model: the #GdaDataModel this row belongs to, or %NULL if the row is outside any data model
+ * gda_row_new:
  * @count: number of #GValue in the new #GdaRow.
  *
  * Creates a #GdaRow which can hold @count #GValue values.
  *
- * The caller of this function is the only owner of a reference to the newly created #GdaRow
- * object, even if @model is not %NULL (it is recommended to pass %NULL as the @model argument
- * if this function is not called from within a #GdaDataModel implementation).
- *
  * Returns: a newly allocated #GdaRow object.
  */
 GdaRow *
@@ -214,16 +209,16 @@ gda_row_new (gint count)
 }
 
 /**
- * gda_row_get_value
+ * gda_row_get_value:
  * @row: a #GdaRow
  * @num: field index.
  *
  * Gets a pointer to a #GValue stored in a #GdaRow.
  *
  * This is a pointer to the internal array of values. Don't try to free
- * or modify it!
+ * or modify it (modifying is reserved to database provider's implementations).
  *
- * Returns: a pointer to the #GValue in the position @num of @row.
+ * Returns: (transfer none): a pointer to the #GValue in the position @num of @row.
  */
 GValue *
 gda_row_get_value (GdaRow *row, gint num)
@@ -234,9 +229,40 @@ gda_row_get_value (GdaRow *row, gint num)
         return & (row->priv->fields[num]);
 }
 
+/**
+ * gda_row_invalidate_value:
+ * @row: a #GdaRow
+ * @value: a #GValue belonging to @row (obtained with gda_row_get_value()).
+ * 
+ * Marks @value as being invalid. This method is mainly used by database
+ * providers' implementations to report any error while reading a value from the database.
+ */
+void
+gda_row_invalidate_value (GdaRow *row, GValue *value)
+{
+	gda_value_set_null (value);
+	G_VALUE_TYPE (value) = G_TYPE_NONE;
+}
 
 /**
- * gda_row_get_length
+ * gda_row_value_is_valid:
+ * @row: a #GdaRow.
+ * @value: a #GValue belonging to @row (obtained with gda_row_get_value()).
+ *
+ * Tells if @value has been marked as being invalid by gda_row_invalidate_value().
+ * This method is mainly used by database
+ * providers' implementations to report any error while reading a value from the database.
+ *
+ * Returns: %TRUE if @value is valid
+ */
+gboolean
+gda_row_value_is_valid (GdaRow *row, GValue *value)
+{
+	return (G_VALUE_TYPE (value) == G_TYPE_NONE) ? FALSE : TRUE;
+}
+
+/**
+ * gda_row_get_length:
  * @row: a #GdaRow.
  *
  * Returns: the number of columns that the @row has.
