@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 2005 - 2008 The GNOME Foundation.
+ * Copyright (C) 2005 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -36,19 +36,19 @@
 #include <libgda/handlers/gda-handler-type.h>
 
 /**
- * gda_server_provider_internal_get_parser
- * @prov:
+ * gda_server_provider_internal_get_parser:
+ * @prov: a #GdaServerProvider
  *
  * This is a factory method to get a unique instance of a #GdaSqlParser object
  * for each #GdaServerProvider object
- * Don't unref() it.
+ * Don't unref it.
  *
- * Returns: a #GdaSqlParser
+ * Returns: (transfer none): a #GdaSqlParser
  */
 GdaSqlParser *
 gda_server_provider_internal_get_parser (GdaServerProvider *prov)
 {
-	
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (prov), NULL);
 	if (prov->priv->parser)
 		return prov->priv->parser;
 	prov->priv->parser = gda_server_provider_create_parser (prov, NULL);
@@ -58,15 +58,15 @@ gda_server_provider_internal_get_parser (GdaServerProvider *prov)
 }
 
 /**
- * gda_server_provider_perform_operation_default
+ * gda_server_provider_perform_operation_default:
  * @provider: a #GdaServerProvider object
- * @cnc: a #GdaConnection object which will be used to perform an action, or %NULL
+ * @cnc: (allow-none): a #GdaConnection object which will be used to perform an action, or %NULL
  * @op: a #GdaServerOperation object
  * @error: a place to store an error, or %NULL
  *
  * Performs the operation described by @op, using the SQL from the rendering of the operation
  *
- * Returns: TRUE if no error occurred
+ * Returns: %TRUE if no error occurred
  */
 gboolean
 gda_server_provider_perform_operation_default (GdaServerProvider *provider, GdaConnection *cnc,
@@ -98,15 +98,15 @@ gda_server_provider_perform_operation_default (GdaServerProvider *provider, GdaC
 }
 
 /**
- * gda_server_provider_get_data_handler_default
+ * gda_server_provider_get_data_handler_default:
  * @provider: a server provider.
- * @cnc: a #GdaConnection object, or %NULL
- * @for_type: a #GType
+ * @cnc: (allow-none): a #GdaConnection object, or %NULL
+ * @type: a #GType
  * @dbms_type: a DBMS type definition
  *
  * Provides the implementation when the default Libgda's data handlers must be used
  * 
- * Returns: a #GdaDataHandler, or %NULL
+ * Returns: (transfer none): a #GdaDataHandler, or %NULL
  */
 GdaDataHandler *
 gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaConnection *cnc,
@@ -123,7 +123,9 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 	    (type == GDA_TYPE_USHORT) ||
 	    (type == G_TYPE_CHAR) ||
 	    (type == G_TYPE_UCHAR) ||
-	    (type == G_TYPE_UINT)) {
+	    (type == G_TYPE_UINT) ||
+	    (type == G_TYPE_LONG) ||
+	    (type == G_TYPE_ULONG)) {
 		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
 		if (!dh) {
 			dh = gda_handler_numerical_new ();
@@ -138,6 +140,8 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_CHAR, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_UCHAR, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_UINT, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_ULONG, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_LONG, NULL);
 			g_object_unref (dh);
 		}
 	}
@@ -168,11 +172,11 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 			g_object_unref (dh);
 		}
 	}
-	else if (type == G_TYPE_ULONG) {
+	else if (type == G_TYPE_GTYPE) {
 		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
 		if (!dh) {
 			dh = gda_handler_type_new ();
-			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_ULONG, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_GTYPE, NULL);
 			g_object_unref (dh);
 		}
 	}
@@ -180,13 +184,89 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 	return dh;
 }
 
+static gboolean
+param_to_null_foreach (GdaSqlAnyPart *part, gpointer data, GError **error)
+{
+	if (part->type == GDA_SQL_ANY_EXPR) {
+		GdaSqlExpr *expr = (GdaSqlExpr*) part;
+		if (expr->param_spec) {
+			GType type = expr->param_spec->g_type;
+			gda_sql_param_spec_free (expr->param_spec);
+			expr->param_spec = NULL;
+
+			if (!expr->value) {
+				if (type != GDA_TYPE_NULL)
+					expr->value = gda_value_new_from_string ("0", type);
+				else
+					g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 0);
+			}
+		}
+	}
+	return TRUE;
+}
 
 /**
- * gda_server_provider_get_schema_nb_columns
- * @schema:
+ * gda_select_alter_select_for_empty:
+ * @stmt: a SELECT #GdaStatement
+ * @error: (allow-none): a place to store errors, or %NULL
+ *
+ * Creates a new #GdaStatement, selecting the same data as @stmt, but which always returns an
+ * empty (no row) data model. This is use dy database providers' implementations.
+ *
+ * Returns: (transfer full): a new #GdaStatement
+ */
+GdaStatement *
+gda_select_alter_select_for_empty (GdaStatement *stmt, GError **error)
+{
+	GdaStatement *estmt;
+	GdaSqlStatement *sqlst;
+	GdaSqlStatementSelect *stsel;
+
+	g_assert (gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT);
+	g_object_get (G_OBJECT (stmt), "structure", &sqlst, NULL);
+	g_assert (sqlst);
+
+	if (sqlst->sql) {
+		g_free (sqlst->sql);
+		sqlst->sql = NULL;
+	}
+	stsel = (GdaSqlStatementSelect*) sqlst->contents;
+
+	/* set the WHERE condition to "1 = 0" */
+	GdaSqlExpr *expr, *cond = stsel->where_cond;
+	GdaSqlOperation *op;
+	if (cond)
+		gda_sql_expr_free (cond);
+	cond = gda_sql_expr_new (GDA_SQL_ANY_PART (stsel));
+	stsel->where_cond = cond;
+	op = gda_sql_operation_new (GDA_SQL_ANY_PART (cond));
+	cond->cond = op;
+	op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+	op->operands = g_slist_prepend (NULL, expr);
+	g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 1);
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+	op->operands = g_slist_prepend (op->operands, expr);
+	g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 0);
+
+	/* replace any selected field which has a parameter with NULL */
+	gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stsel), (GdaSqlForeachFunc) param_to_null_foreach,
+				  NULL, NULL);
+
+	/* create new statement */
+	estmt = g_object_new (GDA_TYPE_STATEMENT, "structure", sqlst, NULL);
+	gda_sql_statement_free (sqlst);
+	return estmt;
+}
+
+/**
+ * gda_server_provider_get_schema_nb_columns:
+ * @schema: a #GdaConnectionSchema
  *
  * Returns: the number of columns the #GdaDataModel for the requested schema
  * must have
+ *
+ * Deprecated: 4.2: This was a leftover from the pre 4.0 area
  */
 gint
 gda_server_provider_get_schema_nb_columns (GdaConnectionSchema schema)
@@ -393,13 +473,15 @@ schema_get_spec (GdaConnectionSchema schema)
 }
 
 /**
- * gda_server_provider_init_schema_model
- * @model:
- * @schema:
+ * gda_server_provider_init_schema_model:
+ * @model: a #GdaDataModel
+ * @schema: a #GdaConnectionSchema
  *
  * Sets the column attributes of @model for the requested schema
  *
- * Returns: TRUE if there was no error
+ * Returns: %TRUE if there was no error
+ *
+ * Deprecated: 4.2: This was a leftover from the pre 4.0 area
  */
 gboolean
 gda_server_provider_init_schema_model (GdaDataModel *model, GdaConnectionSchema schema)
@@ -418,7 +500,7 @@ gda_server_provider_init_schema_model (GdaDataModel *model, GdaConnectionSchema
 	for (i = 0; i < nbcols; i++) {
                 column = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
 
-                gda_column_set_title (column, spec[i].col_name);
+                gda_column_set_description (column, spec[i].col_name);
                 gda_column_set_name (column, spec[i].col_name);
                 gda_column_set_g_type (column, spec[i].data_type);
         }
@@ -427,14 +509,16 @@ gda_server_provider_init_schema_model (GdaDataModel *model, GdaConnectionSchema
 }
 
 /**
- * gda_server_provider_test_schema_model
+ * gda_server_provider_test_schema_model:
  * @model: a #GdaDataModel to test
  * @schema:
  * @error:
  *
  * Test that the structure of @model is correct in regard with @schema
  *
- * Returns: TRUE if @model has the correct structure
+ * Returns: %TRUE if @model has the correct structure
+ *
+ * Deprecated: 4.2: This was a leftover from the pre 4.0 area
  */
 gboolean
 gda_server_provider_test_schema_model (GdaDataModel *model, GdaConnectionSchema schema, GError **error)
@@ -446,7 +530,7 @@ gda_server_provider_test_schema_model (GdaDataModel *model, GdaConnectionSchema
 
 	nbcols = gda_data_model_get_n_columns (model);
 	if (nbcols < gda_server_provider_get_schema_nb_columns (schema)) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, 0, 0, "%s", 
 			     _("Data model for schema has a wrong number of columns"));
 		return FALSE;
 	}
@@ -457,10 +541,10 @@ gda_server_provider_test_schema_model (GdaDataModel *model, GdaConnectionSchema
 
                 column = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
 
-		if (strcmp (gda_column_get_title (column), spec[i].col_name)) {
+		if (strcmp (gda_column_get_description (column), spec[i].col_name)) {
 			g_set_error (error, 0, 0,
 				     _("Data model for schema has a wrong column title: '%s' instead of '%s'"),
-				     gda_column_get_title (column), spec[i].col_name);
+				     gda_column_get_description (column), spec[i].col_name);
 			return FALSE;
 		}
 
@@ -514,172 +598,16 @@ gda_server_provider_handler_declare (GdaServerProvider *prov, GdaDataHandler *dh
 	g_object_ref (dh);
 }
 
-
-/*
- * Find the GType of a DBMS type using the TYPES schema array
- */
-static GType
-find_gtype (GdaDataModel *types, const gchar *name)
-{
-	GType gtype = G_TYPE_INVALID;
-	gint row, nrows;
-	nrows = gda_data_model_get_n_rows (types);
-	for (row = 0; (row < nrows) && (gtype == G_TYPE_INVALID); row++) {
-		const GValue *value;
-		value = gda_data_model_get_value_at (types, 0, row);
-		if (value && gda_value_isa (value, G_TYPE_STRING)) {
-			const gchar *t = g_value_get_string ((GValue*) value);
-			if (!strcmp (t, name)) {
-				value = gda_data_model_get_value_at (types, 3, row);
-				if (value && gda_value_isa (value, G_TYPE_ULONG))
-					gtype = g_value_get_ulong (value);
-			}
-		}
-	}
-	
-	return gtype;
-}
-
-/*
- * Tries to create a list of Key fields for table @table_name using @dict (and if @dict does not know about
- * that table, get the FIELDS schema from the provider)
- *
- * if @types_list is not %NULL, then it will contain a list of GType types of the same length as the returned list.
- */
-static GSList *
-get_key_field_names (GdaConnection *cnc, const gchar *table_name, GSList **types_list, GError **error)
-{
-	GSList *retfields = NULL;
-	if (!table_name || !(*table_name))
-		return NULL;
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return retfields;
-}
-
-/*
- * Tries to create a list of blob fields for table @table_name using @dict (and if @dict does not know about
- * that table, get the FIELDS schema from the provider)
- */
-static GSList *
-get_blob_field_names (GdaConnection *cnc, const gchar *table_name, GError **error)
-{
-	GSList *retfields = NULL;
-	if (!table_name || !(*table_name))
-		return NULL;
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return retfields;
-}
-
 /**
- * gda_server_provider_blob_list_for_update
+ * gda_server_provider_find_file:
+ * @prov: a #GdaServerProvider
+ * @inst_dir: directory where @prov is installed
+ * @filename: name of the file to find
  *
- * Create a SELECT query from an UPDATE query which lists all the BLOB fields in the
- * query which will be updated. This function is used by GdaServerProvider implementations
- * when dealing with BLOB updates.
+ * Finds the location of a @filename. This function should only be used by database provider's
+ * implementations
  *
- * After execution, @out_select contains a new GdaQuery, or %NULL if the update query does not have any
- * BLOB to update.
- *
- * For example UPDATE blobs set name = ##/ *name:'name' type:gchararray* /, data = ##/ *name:'theblob' type:'GdaBlob'* / WHERE id= ##/ *name:'id' type:gint* /
- * will create:
- * SELECT t1.data FROM blobs AS t1 WHERE id= ##/ *name:'id' type:gint* /
- *
- * Returns: TRUE if no error occurred.
- */
-gboolean
-gda_server_provider_blob_list_for_update (GdaConnection *cnc, GdaStatement *query, 
-					  GdaStatement **out_select, GError **error)
-{
-	gboolean retval = TRUE;
-	
-	g_return_val_if_fail (out_select, FALSE);
-	g_return_val_if_fail (GDA_IS_STATEMENT (query), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return retval;
-}
-
-/**
- * gda_server_provider_blob_list_for_delete
- *
- * Create a SELECT query from a DELETE query which lists all the BLOB fields in the
- * query which will be deleted. This function is used by GdaServerProvider implementations
- * when dealing with BLOB deletions.
- *
- * After execution, @out_select contains a new GdaQuery, or %NULL if the delete query does not have any
- * BLOB to delete.
- *
- * For example DELETE FROM blobs WHERE id= ##/ *name:'id' type:gint* /
- * will create:
- * SELECT t1.data FROM blobs AS t1 WHERE id= ##/ *name:'id' type:gint* /
- *
- * Returns: TRUE if no error occurred.
- */
-gboolean
-gda_server_provider_blob_list_for_delete (GdaConnection *cnc, GdaStatement *query, 
-					  GdaStatement **out_stmt, GError **error)
-{
-	gboolean retval = TRUE;
-
-	g_return_val_if_fail (out_stmt, FALSE);
-	g_return_val_if_fail (GDA_IS_STATEMENT (query), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return retval;
-}
-
-/**
- * gda_server_provider_split_update_query
- *
- * When an update query will affect N>1 rows, it can be refined into a new update query which can be executed N times wich
- * will affect one row at a time. This is usefull for providers implementations when dealing with BLOBs where updating
- * rows with a blob can be done only one row at a time.
- *
- * After execution, @out_select contains a new GdaQuery, or %NULL if it is not possible to create the update query.
- *
- * For example UPDATE blobs set name = ##/ *name:'name' type:gchararray* /, data = ##/ *name:'theblob' type:'GdaBlob'* / WHERE name= ##/ *name:'oname' type:gchararray* /
- * will create (if 'id' is a PK of the table to update):
- * UPDATE blobs set name = ##/ *name:'name' type:gchararray* /, data = ##/ *name:'theblob' type:'GdaBlob'* / WHERE id= ##/ *name:'oid' type:gint* /
- *
- * Returns: TRUE if no error occurred.
- */
-gboolean
-gda_server_provider_split_update_query (GdaConnection *cnc, GdaStatement *query, 
-					GdaStatement **out_stmt, GError **error)
-{
-	gboolean retval = TRUE;
-
-	g_return_val_if_fail (out_stmt, FALSE);
-	g_return_val_if_fail (GDA_IS_STATEMENT (query), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return retval;
-}
-
-/**
- * gda_server_provider_select_query_has_blobs
- * 
- * Determines if @query (which must be a SELECT query) returns a data model with some BLOB data
- */
-gboolean
-gda_server_provider_select_query_has_blobs (GdaConnection *cnc, GdaStatement *stmt, GError **error)
-{
-	gboolean has_blobs = FALSE;
-
-	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	TO_IMPLEMENT; /* using @cnc's GdaMetaStore */
-	return has_blobs;
-}
-
-/*
- * finds the location of a @filename
+ * Returns: (transfer full): the complete path to @filename, or %NULL if not found
  */
 gchar *
 gda_server_provider_find_file (GdaServerProvider *prov, const gchar *inst_dir, const gchar *filename)
@@ -712,9 +640,17 @@ gda_server_provider_find_file (GdaServerProvider *prov, const gchar *inst_dir, c
 	return file;
 }
 
-/*
- * Loads and returns the contents of @filename.
- * @filename is searched in several places
+/**
+ * gda_server_provider_load_file_contents:
+ * @inst_dir: directory where the database provider has been installed
+ * @data_dir: DATA directory to look for ($prefix/share)
+ * @filename: name of the file to load
+ *
+ * Loads and returns the contents of @filename, which is searched in several places
+ * This function should only be used by database provider's
+ * implementations
+ *
+ * Returns: (transfer full): a new string containing @filename's contents, or %NULL if not found or if an error occurred
  */
 gchar *
 gda_server_provider_load_file_contents (const gchar *inst_dir, const gchar *data_dir, const gchar *filename)
@@ -737,7 +673,7 @@ gda_server_provider_load_file_contents (const gchar *inst_dir, const gchar *data
 		goto theend;
 	
 	g_free (file);
-	file = g_build_filename (inst_dir, "..", "..", "..", "share", "libgda-3.0", filename, NULL);
+	file = g_build_filename (inst_dir, "..", "..", "..", "share", "libgda-4.0", filename, NULL);
 	if (g_file_get_contents (file, &contents, NULL, NULL))
 		goto theend;
 	contents = NULL;
