@@ -419,11 +419,19 @@ gda_set_new (GSList *holders)
 	return (GdaSet*) obj;
 }
 
-/*
- * _gda_set_new_read_only
+/**
+ * gda_set_new_read_only:
+ * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
+ *
+ * Creates a new #GdaSet like gda_set_new(), but does not allow modifications to any of the #GdaHolder
+ * object in @holders. This function is used for Libgda's database providers' implementation.
+ *
+ * Returns: a new #GdaSet object
+ *
+ * Since: 4.2
  */
 GdaSet *
-_gda_set_new_read_only (GSList *holders)
+gda_set_new_read_only (GSList *holders)
 {
 	GObject *obj;
 
@@ -436,15 +444,13 @@ _gda_set_new_read_only (GSList *holders)
 	return (GdaSet*) obj;
 }
 
-
-
 /**
  * gda_set_copy:
  * @set: a #GdaSet object
  *
  * Creates a new #GdaSet object, copy of @set
  *
- * Returns: a new #GdaSet object
+ * Returns: (transfer full): a new #GdaSet object
  */
 GdaSet *
 gda_set_copy (GdaSet *set)
@@ -477,7 +483,7 @@ gda_set_copy (GdaSet *set)
  * to use an unsupported type will result in a warning, and the returned value holder holding a safe default
  * value.
  *
- * Returns: a new #GdaSet object
+ * Returns: (transfer full): a new #GdaSet object
  */ 
 GdaSet *
 gda_set_new_inline (gint nb, ...)
@@ -574,13 +580,13 @@ gda_set_new_inline (gint nb, ...)
 /**
  * gda_set_set_holder_value:
  * @set: a #GdaSet object
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
  * @...: value, of the correct type, depending on the requested holder's type (not NULL)
  *
  * Set the value of the #GdaHolder which ID is @holder_id to a specified value
  *
- * Returns: TRUE if no error occurred and the value was set correctly
+ * Returns: %TRUE if no error occurred and the value was set correctly
  */
 gboolean
 gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, ...)
@@ -659,7 +665,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
  *
  * Get the value of the #GdaHolder which ID is @holder_id
  *
- * Returns: the requested GValue, or %NULL (see gda_holder_get_value())
+ * Returns: (transfer none): the requested GValue, or %NULL (see gda_holder_get_value())
  */
 const GValue *
 gda_set_get_holder_value (GdaSet *set, const gchar *holder_id)
@@ -702,12 +708,12 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
 /**
  * gda_set_new_from_spec_string:
  * @xml_spec: a string
- * @error: a place to store the error, or %NULL
+ * @error: (allow-none): a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: a new object, or %NULL if an error occurred
+ * Returns: (transfer full): a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
@@ -794,12 +800,12 @@ gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
 /**
  * gda_set_new_from_spec_node:
  * @xml_spec: a #xmlNodePtr for a &lt;holders&gt; tag
- * @error: a place to store the error, or %NULL
+ * @error: (allow-none): a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: a new object, or %NULL if an error occurred
+ * Returns: (transfer full): a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
@@ -977,9 +983,8 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
 						      G_CALLBACK (att_holder_changed_cb), set);
 	}
-	if (gda_holder_get_g_type (holder) == GDA_TYPE_NULL)
-		g_signal_handlers_disconnect_by_func (holder,
-						      G_CALLBACK (holder_notify_cb), set);
+	g_signal_handlers_disconnect_by_func (holder,
+					      G_CALLBACK (holder_notify_cb), set);
 
 	/* now destroy the GdaSetNode and the GdaSetSource if necessary */
 	node = gda_set_get_node (set, holder);
@@ -1255,10 +1260,30 @@ holder_notify_cb (GdaHolder *holder, GParamSpec *pspec, GdaSet *dataset)
 {
 	GType gtype;
 	gtype = gda_holder_get_g_type (holder);
-	g_assert (gtype != GDA_TYPE_NULL);
-	g_signal_handlers_disconnect_by_func (holder,
-					      G_CALLBACK (holder_notify_cb), dataset);
-	g_signal_emit (dataset, gda_set_signals[HOLDER_TYPE_SET], 0, holder);
+	if (!strcmp (pspec->name, "g-type")) {
+		g_assert (gtype != GDA_TYPE_NULL);
+		g_signal_emit (dataset, gda_set_signals[HOLDER_TYPE_SET], 0, holder);
+	}
+	else if (!strcmp (pspec->name, "name")) {
+#ifdef GDA_DEBUG_signal
+	g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (dataset), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder,
+		       GDA_ATTRIBUTE_NAME, gda_holder_get_attribute (holder, GDA_ATTRIBUTE_NAME));
+#ifdef GDA_DEBUG_signal
+	g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else if (!strcmp (pspec->name, "description")) {
+#ifdef GDA_DEBUG_signal
+	g_print (">> 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (dataset), gda_set_signals[HOLDER_ATTR_CHANGED], 0, holder,
+		       GDA_ATTRIBUTE_DESCRIPTION, gda_holder_get_attribute (holder, GDA_ATTRIBUTE_DESCRIPTION));
+#ifdef GDA_DEBUG_signal
+	g_print ("<< 'HOLDER_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
 }
 
 static gboolean
@@ -1300,7 +1325,10 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 		if (gda_holder_get_g_type (holder) == GDA_TYPE_NULL)
 			g_signal_connect (G_OBJECT (holder), "notify::g-type",
 					  G_CALLBACK (holder_notify_cb), set);
-
+		g_signal_connect (G_OBJECT (holder), "notify::name",
+				  G_CALLBACK (holder_notify_cb), set);
+		g_signal_connect (G_OBJECT (holder), "notify::description",
+				  G_CALLBACK (holder_notify_cb), set);
 		return TRUE;
 	}
 	else if (similar == holder)
@@ -1362,7 +1390,7 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 /**
  * gda_set_is_valid:
  * @set: a #GdaSet object
- * @error: a place to store validation errors, or %NULL
+ * @error: (allow-none): a place to store validation errors, or %NULL
  *
  * This method tells if all @set's #GdaHolder objects are valid, and if
  * they represent a valid combination of values, as defined by rules
