@@ -41,7 +41,7 @@ static gboolean     gda_sqlite_handler_bin_accepts_g_type       (GdaDataHandler
 static const gchar *gda_sqlite_handler_bin_get_descr              (GdaDataHandler *dh);
 
 struct  _GdaSqliteHandlerBinPriv {
-	gchar             *detailled_descr;
+	gchar             *detailed_descr;
 	guint              nb_g_types;
 	GType             *valid_g_types;
 };
@@ -76,7 +76,7 @@ _gda_sqlite_handler_bin_get_type (void)
 
 		g_static_mutex_lock (&registering);
 		if (type == 0) {
-			type = g_type_register_static (G_TYPE_OBJECT, "GdaSqliteHandlerBin", &info, 0);
+			type = g_type_register_static (G_TYPE_OBJECT, CLASS_PREFIX "HandlerBin", &info, 0);
 			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
 		}
 		g_static_mutex_unlock (&registering);
@@ -112,7 +112,7 @@ gda_sqlite_handler_bin_init (GdaSqliteHandlerBin * hdl)
 {
 	/* Private structure */
 	hdl->priv = g_new0 (GdaSqliteHandlerBinPriv, 1);
-	hdl->priv->detailled_descr = _("SQlite binary handler");
+	hdl->priv->detailed_descr = _("SQlite binary handler");
 	hdl->priv->nb_g_types = 1;
 	hdl->priv->valid_g_types = g_new0 (GType, hdl->priv->nb_g_types);
 	hdl->priv->valid_g_types[0] = GDA_TYPE_BINARY;
