@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2009 The GNOME Foundation.
+ * Copyright (C) 2009 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -123,6 +123,7 @@ job_free (Job *job)
 struct _SignalSpec {
         GSignalQuery  sigprop; /* must be first */
 
+	gboolean      private;
 	GThread      *worker_thread;
 	GAsyncQueue  *reply_queue; /* a ref is held here */
 
@@ -195,6 +196,7 @@ get_thread_data (GdaThreadWrapper *wrapper, GThread *thread)
 		td = g_new0 (ThreadData, 1);
 		td->owner = thread;
 		td->from_worker_thread = g_async_queue_new_full ((GDestroyNotify) job_free);
+		td->jobs = NULL;
 		td->results = NULL;
 
 		g_hash_table_insert (wrapper->priv->threads_hash, thread, td);
@@ -207,10 +209,12 @@ static void
 thread_data_free (ThreadData *td)
 {
 	g_async_queue_unref (td->from_worker_thread);
+	td->from_worker_thread = NULL;
 	g_assert (!td->jobs);
 	if (td->results) {
 		g_slist_foreach (td->results, (GFunc) job_free, NULL);
 		g_slist_free (td->results);
+		td->results = NULL;
 	}
 	if (td->signals_list) {
 		GSList *list;
@@ -245,13 +249,6 @@ static void gda_thread_wrapper_get_property (GObject *object,
 					     GValue *value,
 					     GParamSpec *pspec);
 
-enum {
-	DUMMY,
-	LAST_SIGNAL
-};
-
-static gint gda_thread_wrapper_signals[LAST_SIGNAL] = { 0 };
-
 /* properties */
 enum {
 	PROP_0
@@ -320,7 +317,6 @@ worker_thread_entry_point (GAsyncQueue *to_worker_thread)
 				job->u.exe.result = NULL;
 				job->void_func (job->arg, &(job->u.exe.error));
 			}
-			/*g_print ("... done job %d\n", job->job_id);*/
 			g_async_queue_push (job->reply_queue, job);
 		}
 		else
@@ -567,6 +563,7 @@ gda_thread_wrapper_execute (GdaThreadWrapper *wrapper, GdaThreadWrapperFunc func
                         job->u.exe.result = NULL;
                         job->void_func (job->arg, &(job->u.exe.error));
                 }
+		/* g_print ("... IMMEDIATELY done job %d => %p\n", job->job_id, job->u.exe.result); */
                 g_async_queue_push (job->reply_queue, job);
         }
         else
@@ -628,7 +625,7 @@ gda_thread_wrapper_execute_void (GdaThreadWrapper *wrapper, GdaThreadWrapperVoid
 	job->reply_queue = g_async_queue_ref (td->from_worker_thread);
 
 	id = job->job_id;
-	/*g_print ("... submitted VOID job %d\n", id);*/
+	/* g_print ("... submitted VOID job %d\n", id); */
 
 	td->jobs = g_slist_append (td->jobs, job);
 
@@ -640,6 +637,7 @@ gda_thread_wrapper_execute_void (GdaThreadWrapper *wrapper, GdaThreadWrapperVoid
                         job->u.exe.result = NULL;
                         job->void_func (job->arg, &(job->u.exe.error));
                 }
+		/* g_print ("... IMMEDIATELY done VOID job %d => %p\n", job->job_id, job->u.exe.result); */
                 g_async_queue_push (job->reply_queue, job);
         }
         else
@@ -743,7 +741,6 @@ gda_thread_wrapper_iterate (GdaThreadWrapper *wrapper, gboolean may_block)
 		job = g_async_queue_try_pop (td->from_worker_thread);
 	if (job) {
 		gboolean do_again = FALSE;
-
 		td->jobs = g_slist_remove (td->jobs, job);
 
 		if (job->type == JOB_TYPE_EXECUTE) {
@@ -758,16 +755,16 @@ gda_thread_wrapper_iterate (GdaThreadWrapper *wrapper, gboolean may_block)
 			/* run callback now */
 			SignalSpec *spec = job->u.signal.spec;
 
-			signal_spec_lock (spec);
 			if (spec->callback)
 				spec->callback (wrapper, spec->instance, ((GSignalQuery*)spec)->signal_name,
 						job->u.signal.n_param_values, job->u.signal.param_values, NULL,
 						spec->data);
 			else
 				g_print ("Not propagating signal %s\n", ((GSignalQuery*)spec)->signal_name);
-			signal_spec_unref (spec);
 			job->u.signal.spec = NULL;
 			job_free (job);
+			signal_spec_lock (spec);
+			signal_spec_unref (spec);
 			do_again = TRUE;
 		}
 		else
@@ -920,8 +917,8 @@ worker_thread_closure_marshal (GClosure *closure,
 	if (g_thread_self () !=  sigspec->worker_thread)
 		return;
 
-	/* check that the worker thread is working on a job for which job->reply_queue == sigspec->reply_queue */
-	if (g_static_private_get (&worker_thread_current_queue) != sigspec->reply_queue)
+	/* check that the worker thread is working on a job for which job->reply_queue == sigspec->reply_queue */	if (sigspec->private &&
+	    g_static_private_get (&worker_thread_current_queue) != sigspec->reply_queue)
 		return;
 
 	gint i;
@@ -952,11 +949,62 @@ worker_thread_closure_marshal (GClosure *closure,
 	signal_spec_unlock (sigspec);
 }
 
+/* 
+ * Executed in sub thread (or potentially in other threads, in which case will be ignored)
+ * pushes data into the queue 
+ */
+static void
+worker_thread_closure_marshal_anythread (GClosure *closure,
+					 GValue *return_value,
+					 guint n_param_values,
+					 const GValue *param_values,
+					 gpointer invocation_hint,
+					 gpointer marshal_data)
+{
+	SignalSpec *sigspec = (SignalSpec *) closure->data;
+
+	gint i;
+	/*
+	  for (i = 1; i < n_param_values; i++) {
+		g_print ("\t%d => %s\n", i, gda_value_stringify (param_values + i));
+	}
+	*/
+	Job *job= g_new0 (Job, 1);
+	job->type = JOB_TYPE_SIGNAL;
+	job->u.signal.spec = signal_spec_ref (sigspec);
+	job->u.signal.n_param_values = n_param_values - 1;
+	job->u.signal.param_values = g_new0 (GValue, job->u.signal.n_param_values);
+	for (i = 1; i < n_param_values; i++) {
+		const GValue *src;
+		GValue *dest;
+
+		src = param_values + i;
+		dest = job->u.signal.param_values + i - 1;
+
+		if (G_VALUE_TYPE (src) != GDA_TYPE_NULL) {
+			g_value_init (dest, G_VALUE_TYPE (src));
+			g_value_copy (src, dest);
+		}
+	}
+
+	g_async_queue_push (sigspec->reply_queue, job);
+	signal_spec_unlock (sigspec);
+}
+
 /**
  * gda_thread_wrapper_connect_raw
  * @wrapper: a #GdaThreadWrapper object
  * @instance: the instance to connect to
  * @sig_name: a string of the form "signal-name::detail"
+ * @private_thread:  set to %TRUE if @callback is to be invoked only if the signal has
+ *    been emitted while in @wrapper's private sub thread (ie. used when @wrapper is executing some functions
+ *    specified by gda_thread_wrapper_execute() or gda_thread_wrapper_execute_void()), and to %FALSE if the
+ *    callback is to be invoked whenever the signal is emitted, independently of th thread in which the
+ *    signal is emitted.
+ * @private_job: set to %TRUE if @callback is to be invoked only if the signal has
+ *    been emitted when a job created for the calling thread is being executed, and to %FALSE
+ *    if @callback has to be called whenever the @sig_name signal is emitted by @instance. Note that
+ *    this argument is not taken into account if @private_thread is set to %FALSE.
  * @callback: a #GdaThreadWrapperCallback function
  * @data: data to pass to @callback's calls
  *
@@ -968,10 +1016,10 @@ worker_thread_closure_marshal (GClosure *closure,
  *  <listitem><para>the signal handler must not have to return any value</para></listitem>
  *  <listitem><para>the @callback function will be called asynchronously, the caller may need to use 
  *    gda_thread_wrapper_iterate() to get the notification</para></listitem>
- *  <listitem><para>the @callback function will be called only if the signal has been emitted by @instance 
- *    while being used in @wrapper's private sub thread (ie. used when @wrapper is executing some functions
- *    specified by
- *    gda_thread_wrapper_execute() or gda_thread_wrapper_execute_void())</para></listitem>
+ *  <listitem><para>if @private is set to %TRUE, then the @callback function will be called only
+ *    if the signal has been emitted by @instance while doing a job on behalf of the current
+ *    calling thread. If @private is set to %FALSE, then @callback will be called whenever @instance
+ *    emits the @sig_name signal.</para></listitem>
  * </itemizedlist>
  *
  * Also note that signal handling is done asynchronously: when emitted in the worker thread, it
@@ -990,6 +1038,7 @@ gulong
 gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
 				gpointer instance,
 				const gchar *sig_name,
+				gboolean private_thread, gboolean private_job,
 				GdaThreadWrapperCallback callback,
 				gpointer data)
 {
@@ -1012,6 +1061,7 @@ gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
         }
 
         sigspec = g_new0 (SignalSpec, 1);
+	sigspec->private = private_job;
         g_signal_query (sigid, (GSignalQuery*) sigspec);
 
 	if (((GSignalQuery*) sigspec)->return_type != G_TYPE_NONE) {
@@ -1029,7 +1079,10 @@ gda_thread_wrapper_connect_raw (GdaThreadWrapper *wrapper,
 
 	GClosure *cl;
 	cl = g_closure_new_simple (sizeof (GClosure), sigspec);
-	g_closure_set_marshal (cl, (GClosureMarshal) worker_thread_closure_marshal);
+	if (private_thread)
+		g_closure_set_marshal (cl, (GClosureMarshal) worker_thread_closure_marshal);
+	else
+		g_closure_set_marshal (cl, (GClosureMarshal) worker_thread_closure_marshal_anythread);
 	sigspec->signal_id = g_signal_connect_closure (instance, sig_name, cl, FALSE);
 
 	td->signals_list = g_slist_append (td->signals_list, sigspec);
@@ -1107,9 +1160,9 @@ gda_thread_wrapper_disconnect (GdaThreadWrapper *wrapper, gulong id)
 	td->signals_list = g_slist_remove (td->signals_list, sigspec);
 	g_signal_handler_disconnect (sigspec->instance, sigspec->signal_id);
 	sigspec->instance = NULL;
-	sigspec->signal_id = NULL;
+	sigspec->signal_id = 0;
 	g_async_queue_unref (sigspec->reply_queue);
-	sigspec->reply_queue = 0;
+	sigspec->reply_queue = NULL;
 	sigspec->callback = NULL;
 	sigspec->data = NULL;
 	signal_spec_unref (sigspec);
@@ -1156,6 +1209,9 @@ gda_thread_wrapper_steal_signal (GdaThreadWrapper *wrapper, gulong id)
 		return;
 	}
 
+	if (old_td->owner == g_thread_self ())
+		return;
+
         /* merge old_td and new_td */
         if (old_td->signals_list) {
                 GSList *list;
@@ -1165,6 +1221,8 @@ gda_thread_wrapper_steal_signal (GdaThreadWrapper *wrapper, gulong id)
 				new_td = get_thread_data (wrapper, g_thread_self ());
 				new_td->signals_list = g_slist_prepend (new_td->signals_list, sigspec);
 				old_td->signals_list = g_slist_remove (old_td->signals_list, sigspec);
+				g_async_queue_unref (sigspec->reply_queue);
+				sigspec->reply_queue = g_async_queue_ref (new_td->from_worker_thread);
 				break;
 			}
                 }
